@@ -225,51 +225,59 @@ sub runCommand {
     return ($exitcode, $buff);
 }
 
-sub _quotient {
-    my($index, $encoded) = @_;
-
-    # Same as $index * 256 + $product_key ???
-    my $dividend = $index * 256 ^ $encoded; ## no critic (ProhibitBitwise)
-
-    # return modulus and integer quotient
-    return(
-        $dividend % 24,
-        $dividend / 24,
-    );
-}
-
-#http://www.perlmonks.org/?node_id=497616
-# Thanks William Gannon && Charles Clarkson
+# inspired by http://poshcode.org/4363
 sub parseProductKey {
-    my ($key) = @_;
-    return unless $key;
+    my ($key_string) = @_;
 
-    my @encoded = ( unpack 'C*', $key )[ reverse 52 .. 66 ];
+    ## no critic (ProhibitBitwise)
 
-    # Get indices
-    my @indices;
-    foreach ( 0 .. 24 ) {
+    return unless $key_string;
+
+    my @key_bytes = unpack 'C*', $key_string;
+
+    # check for Windows 8/Office 2013 style key (can contains the letter "N")
+    my $containsN  = ($key_bytes[66] >> 3) & 1;
+    $key_bytes[66] = ($key_bytes[66] & 0xF7);
+
+    # length of product key, in chars
+    my $chars_length = 25;
+
+    # length of product key, in bytes
+    my $bytes_length = 15;
+
+    # product key available characters
+    my @letters = qw(B C D F G H J K M P Q R T V W X Y 2 3 4 6 7 8 9);
+
+    # extract bytes 52 to 66
+    my @bytes = @key_bytes[52 .. 66];
+
+    # decoded product key
+    my @chars;
+
+    for (my $i = $chars_length - 1; $i >= 0; $i--) {
         my $index = 0;
-
-        # Shift off remainder
-        ( $index, $_ ) = _quotient( $index, $_ ) foreach @encoded;
-
-        # Store index.
-        unshift @indices, $index;
+        for (my $j = $bytes_length - 1; $j >= 0; $j--) {
+            my $value = ($index << 8) | $bytes[$j];
+            $bytes[$j] = $value / scalar @letters;
+            $index = $value % (scalar @letters);
+        }
+        $chars[$i] = $letters[$index];
     }
 
-    # translate base 24 "digits" to characters
-    my $cd_key =
-        join '',
-        qw( B C D F G H J K M P Q R T V W X Y 2 3 4 6 7 8 9 )[ @indices ];
+    if ($containsN != 0) {
+        my $first_char = shift @chars;
+        my $first_char_index = 0;
+        for (my $index = 0; $index < scalar @letters; $index++) {
+            next if $first_char ne $letters[$index];
+            $first_char_index = $index;
+            last;
+        }
 
-    # Add seperators
-    $cd_key =
-        join '-',
-        $cd_key =~ /(.{5})/g;
+        splice @chars, $first_char_index, 0, 'N';
+    }
 
-    return if $cd_key =~ /^[B-]*$/;
-    return $cd_key;
+    return sprintf
+        '%s%s%s%s%s-%s%s%s%s%s-%s%s%s%s%s-%s%s%s%s%s-%s%s%s%s%s', @chars;
 }
 
 sub getInterfaces {
